import 'dart:async';
import 'dart:io';
import 'package:diacritic/diacritic.dart';
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'package:sembast/sembast_io.dart';
import 'package:natinfo_flutter/api/swagger.swagger.dart';

class _Scored {
    final Natinf natinf;
    final int relevance;
    final int qualificationPos;
    final int pk;

    _Scored({
      required this.natinf,
      required this.relevance,
      required this.qualificationPos,
      required this.pk,
    });
  }

class DatabaseHelper {
  static final DatabaseHelper _singleton = DatabaseHelper._internal();
  factory DatabaseHelper() => _singleton;
  DatabaseHelper._internal();

  Database? _db;
  final _store = intMapStoreFactory.store('natinf');
  final _historyStore = intMapStoreFactory.store('historique');

  /// Initialise la base de données Sembast
  Future<void> initDb() async {
    final Directory dir = await getApplicationDocumentsDirectory();
    final dbPath = join(dir.path, 'natinfo.db');
    _db = await databaseFactoryIo.openDatabase(dbPath);
  }

  Future<int> insertOrUpdateNatinf(Natinf natinf) async {
    final record = _store.record(natinf.id!);
    await record.put(_db!, natinf.toJson());
    return natinf.id!;
  }

  Future<Natinf?> getNatinfByNumero(String numero) async {
    if (_db == null) throw Exception("La base de données n'est pas initialisée.");
    final finder = Finder(filter: Filter.equals('numero_natinf', numero));
    final record = await _store.findFirst(_db!, finder: finder);
    if (record != null) {
      return Natinf.fromJson(record.value);
    }
    return null;
  }
  
  /// Recherche locale en essayant de reproduire la logique de l'API NATINFo
  Future<List<Natinf>> getNatinfByQuery(String query) async {
    String normalize(String s) => removeDiacritics(s)
        .toLowerCase()
        .replaceAll(RegExp(r'[^\w]+'), ' ')
        .trim();

    final normalizedQuery = normalize(query);
    final queryWords = normalizedQuery.split(' ').where((w) => w.isNotEmpty).toList();

    RegExp makePattern(String kw) {
      var p = RegExp.escape(kw);
      p = p.replaceAllMapped(
        RegExp(r'([A-Za-z])\\?\.?([0-9])'),
        (m) => '${m[1]}\\.?${m[2]}',
      );
      return RegExp(r'\b' + p, caseSensitive: false);
    }

    final List<RegExp> patterns = [];
    for (final word in queryWords) {
      patterns.add(makePattern(word));
    }

    final fullQueryRegex =
        RegExp(r'\b' + RegExp.escape(normalizedQuery) + r'\b', caseSensitive: false);

    final records = await _store.find(_db!);
    final buffer = <_Scored>[];

    for (final record in records) {
      final map = record.value as Map<String, dynamic>;

      final numero         = normalize(map['numero_natinf']?.toString() ?? '');
      final nature         = normalize(map['nature_infraction']?.toString() ?? '');
      final qualification  = normalize(map['qualification_infraction']?.toString() ?? '');
      final definiePar     = normalize(map['definie_par']?.toString() ?? '');
      final reprimeePar    = normalize(map['reprimee_par']?.toString() ?? '');

      final bool allMatch = patterns.every((pat) =>
          pat.hasMatch(numero) ||
          pat.hasMatch(nature) ||
          pat.hasMatch(qualification) ||
          pat.hasMatch(definiePar) ||
          pat.hasMatch(reprimeePar));

      if (!allMatch) continue;

      int relevance = 0;
      if (fullQueryRegex.hasMatch(numero)) {
        relevance = 100;
      } else if (fullQueryRegex.hasMatch(qualification)) {
        relevance = 90;
      } else if (fullQueryRegex.hasMatch(definiePar)) {
        relevance = 80;
      } else if (fullQueryRegex.hasMatch(reprimeePar)) {
        relevance = 70;
      }

      int qualificationPos = qualification.indexOf(normalizedQuery);
      if (qualificationPos == -1) qualificationPos = 100000;

      buffer.add(_Scored(
        natinf: Natinf.fromJson(map),
        relevance: relevance,
        qualificationPos: qualificationPos,
        pk: record.key,
      ));
    }

    buffer.sort((a, b) {
      final rel = b.relevance.compareTo(a.relevance);
      if (rel != 0) return rel;

      final pos = a.qualificationPos.compareTo(b.qualificationPos);
      if (pos != 0) return pos;

      return a.pk.compareTo(b.pk);
    });

    return buffer.map((s) => s.natinf).toList();
  }

  Future<int> addHistory(String numero) async {
    if (_db == null) throw Exception("La base de données n'est pas initialisée.");
    return await _historyStore.add(_db!, {
      'numero_natinf': numero,
      'opened_at': DateTime.now().millisecondsSinceEpoch,
    });
  }

  Future<List<Map<String, dynamic>>> getHistory() async {
    if (_db == null) throw Exception("La base de données n'est pas initialisée.");
    final finder = Finder(sortOrders: [SortOrder('opened_at', false)]);
    final records = await _historyStore.find(_db!, finder: finder);
    return records.map((record) => record.value as Map<String, dynamic>).toList();
  }

  Future<void> clearHistory() async {
    if (_db == null) throw Exception("La base de données n'est pas initialisée.");
    await _historyStore.delete(_db!);
  }

  Future<void> clearOfflineData() async {
    if (_db == null) throw Exception("La base de données n'est pas initialisée.");
    await _store.delete(_db!);
    print("Données locales supprimées.");
  }

  Future<bool> isEmpty() async {
    if (_db == null) {
      throw Exception("La base de données n'est pas initialisée.");
    }
    final count = await _store.count(_db!);
    return count == 0;
  }
}